<?php
	function prepare_header()
	{
		if( !function_exists('apache_request_headers') ) {
			return false;
		}
		$raw_header = apache_request_headers();

		if(isset($raw_header['Authorization']))
		{
			$raw_header = explode(',', $raw_header['Authorization']);
			
			foreach($raw_header as $k=>$v)
			{
				$raw_header[$k] = preg_replace('/"/', '', $raw_header[$k]);	
				$raw_header[$k] = explode('=', $raw_header[$k]);
				$ready_header[strtolower(trim($raw_header[$k][0]))] = $raw_header[$k][1]; 
			}
			unset($raw_header);
			
			return $ready_header;
		}else return false;
	}
	
	function check_if_basic_auth()
	{
		if( !function_exists('apache_request_headers') ) {
			return false;
		}
		$raw_header = apache_request_headers(); 
		if( !$raw_header || !isset($raw_header['Authorization']) ) {
			return false;
		}
		if( ! preg_match('/^Basic\s(.*)$/iu', $raw_header['Authorization'], $m) ) {
			return false;
		}
		$tmp	= @base64_decode( trim($m[1]) );
		if( ! $tmp || ! preg_match('/^([^\:]+)\:(.*)$/iu', $tmp, $m) ) {
			return false;
		}
		return array( trim($m[1]), trim($m[2]) );
	}
	function detect_app()
	{
		if( !function_exists('apache_request_headers') ) {
			return 1;
		}
		
		if(preg_match('/TweetDeck/iu', implode(' ', apache_request_headers()))) return 5;
		
		return 1;
	}	
	function is_valid_url($link)
	{
		if(!preg_match('/^(http|https):\/\/((([a-z0-9.-]+\.)+[a-z]{2,4})|([0-9\.]{1,4}){4})(\/([a-z-0-9-_\\:%\.\?\!\=\+\&\/\#\~\;\,\@]+)?)?$/', $link)) 
			return FALSE;
		else return TRUE;
	}
	function is_valid_date($date)
	{
		if(preg_match ("/^([0-9]{4})-([0-9]{2})-([0-9]{2})$/", $date))
		{
			$arr = split("-", $date); 
			if(intval(date("Y", time())) < intval($arr[0]) || intval($arr[0]) < 1900) return false;
		      if (is_numeric($arr[0]) && is_numeric($arr[1]) && is_numeric($arr[2])) return checkdate($arr[1], $arr[2], $arr[0]);
		        else return false;
		        
		}else return false;
	}
	function utc_offset()
	{
	      $diff = gmmktime(0,0,0,12,29,2007) - mktime(0,0,0,12,29,2007);
		$sign = ( ( $diff >= 0 ) ? '+' : '-');
		$diff = abs($diff);
		$hours = str_pad(strval(floor($diff/3600)), 2, '0',STR_PAD_LEFT);
		$minutes = str_pad(strval(floor($diff/60) % 60), 2, '0',STR_PAD_LEFT);
		
		return $sign.$hours.$minutes.'0';
	}
	
 	function get_call_result($query, $format)
	{
		global $db2;
		$res = $db2->query($query);
			
		if($db2->num_rows($res) > 0) 
		{
			$row = (array) $db2->fetch_object($res);
			$answer = '';
			if($format == 'xml' || $format == 'rss' || $format == 'atom')
			{
				foreach($row as $tag => $value)
				{
					$value = ($value == '') ? false: $value;
					$answer .= '<'.$tag.'>'.$value.'</'.$tag.'>';
				}
			}elseif($format == 'json')
			{
				foreach($row as $tag => $value)
				{
					$value = ($value == '') ? false: '"'.$value.'"';
					$answer .= '"'.$tag.'": '.$value.',';
				}
				$answer = substr($answer,0,-1);
			}
				
		}else return false;
		
		return $answer;
	}
	function print_status($status_id, $format='xml', $q=FALSE, $post_type = 'public')
	{
		global $db2, $C;
		require_once($C->INCPATH.'classes/class_post.php');
		
		if($q)
		{
			$res = $db2->query($q);
			if(!$db2->num_rows($res)) return false;
			$res = $db2->fetch_object($res);
			$status_id = $res->pid;
		}
		$status = new post($post_type, $status_id); 	
		
		$answer = '';
		
		if(!isset($status->post_id)) return $answer;
		
		if($format == 'xml' || $format == 'rss' || $format == 'atom')
		{ 
			$answer .= '<created_at>'.gmdate('D M d H:i:s \+0000 Y',$status->post_date).'</created_at>';
			$answer .= '<id>'.$status->post_id.'</id>';
			$answer .= '<text>'.$status->post_message.'</text>';
			$answer .= '<source>'.$C->SITE_TITLE.'</source>';
			$answer .= '<truncated>false</truncated>';
			$answer .= '<in_reply_to_status_id>false</in_reply_to_status_id>';
			$answer .= '<in_reply_to_user_id>false</in_reply_to_user_id>';
			$answer .= '<favorited>false</favorited>';
			$answer .= '<in_reply_to_screen_name>false</in_reply_to_screen_name>';
			
		}elseif($format == 'json')
		{			
			$answer .= '"created_at": "'.gmdate('D M d H:i:s \+0000 Y', $status->post_date).'",';
			$answer .= '"id": "'.$status->post_id.'",';
			$answer .= '"text": "'.$status->post_message.'",';	
			$answer .= '"source": "'.$C->SITE_TITLE.'",';
			$answer .= '"truncated": "false",';
			$answer .= '"in_reply_to_status_id": "false",';
			$answer .= '"in_reply_to_user_id": "false",';
			$answer .= '"favorited": "false",';
			$answer .= '"in_reply_to_screen_name": "false"';
		}		
		return $answer;	
	}
	
	function print_user($id, $format = 'xml')
	{
		global $network, $db2, $C;
		$img_path = $C->SITE_URL.'i/avatars/thumbs1/';
		
		$u = $network->get_user_by_id($id);		
		
		$info	= $network->get_user_follows($id);
		if(!$info) $friends = 0;
		
		$following	= array_keys($info->follow_users);
		$friends = count($following);
		
		$favorites = $db2->query('SELECT COUNT(user_id) AS num FROM post_favs WHERE user_id ='.$id);
		$favorites = $db2->fetch_object($favorites);
		$favorites = $favorites->num;
		
		$answer = '';
		$img_check = ($u->avatar != '')? $u->avatar:'_noavatar_user.gif';
		$active = ($u->active==1) ? 'true':'false';
		if($format == 'xml' || $format == 'rss' || $format == 'atom')
		{ 
			$answer .= '<id>'.$u->id.'</id>';
			$answer .= '<name>'.$u->fullname.'</name>';
			$answer .= '<screen_name>'.$u->username.'</screen_name>';
			$answer .= '<location>'.$u->location.'</location>';
			$answer .= '<description>'.$u->about_me.'</description>';
			$answer .= '<profile_image_url>'.$img_path.$img_check.'</profile_image_url>';
			$answer .= '<url>false</url>';
			$answer .= '<protected>false</protected>';
			$answer .= '<followers_count>'.$u->num_followers.'</followers_count>';
			$answer .= '<profile_background_color>false</profile_background_color>';
			$answer .= '<profile_text_color>false</profile_text_color>';
			$answer .= '<profile_link_color>false</profile_link_color>';
			$answer .= '<profile_sidebar_fill_color>false</profile_sidebar_fill_color>';
			$answer .= '<profile_sidebar_border_color>false</profile_sidebar_border_color>';
			$answer .= '<friends_count>'.$friends.'</friends_count>';
			$answer .= '<created_at>'.gmdate('D M d H:i:s \+0000 Y',$u->reg_date).'</created_at>';
			$answer .= '<favourites_count>'.$favorites.'</favourites_count>';
			$answer .= '<utc_offset>'.utc_offset().'</utc_offset>';
			$answer .= '<timezone>'.$u->timezone.'</timezone>';		
			$answer .= '<profile_background_image_url>false</profile_background_image_url>';
			$answer .= '<profile_background_tile>false</profile_background_tile>';
			$answer .= '<statuses_count>'.$u->num_posts.'</statuses_count>';	
			$answer .= '<notifications>false</notifications>';
			$answer .= '<following>false</following>';
			$answer .= '<verified>'.$active.'</verified>';
			
		}elseif($format == 'json')
		{			
			$answer .= '"id": "'.$u->id.'",';
			$answer .= '"name": "'.$u->fullname.'",';
			$answer .= '"screen_name": "'.$u->username.'",';
			$answer .= '"location": "'.$u->location.'",';
			$answer .= '"description": "'.$u->about_me.'",';
			$answer .= '"profile_image_url": "'.$img_path.$img_check.'",';
			$answer .= '"url": "false",';
			$answer .= '"protected": "false",';
			$answer .= '"followers_count": "'.$u->num_followers.'",';
			$answer .= '"profile_background_color": "false",';
			$answer .= '"profile_text_color": "false",';
			$answer .= '"profile_link_color": "false",';
			$answer .= '"profile_sidebar_fill_color": "false",';
			$answer .= '"profile_sidebar_border_color": "false",';
			$answer .= '"friends_count": "'.$friends.'",';
			$answer .= '"created_at": "'.gmdate('D M d H:i:s \+0000 Y',$u->reg_date).'",';
			$answer .= '"favourites_count": "'.$favorites.'",';
			$answer .= '"utc_offset": "'.utc_offset().'",';
			$answer .= '"timezone": "'.$u->timezone.'",';
			$answer .= '"profile_background_image_url": "false",';
			$answer .= '"profile_background_tile": "false",';	
			$answer .= '"statuses_count": "'.$u->num_posts.'",';
			$answer .= '"notifications": "false",';
			$answer .= '"following": "false",';
			$answer .= '"verified": "'.$active.'"';
		}
		
		return $answer;
	}
	function print_status_simple($post_id, $data_format, $post_type='public')
	{
		global $db2, $C;
		require_once($C->INCPATH.'classes/class_post.php');

		$status = new post($post_type, $post_id); 	
		
		$answer = '';
		
		if(!isset($status->post_id)) return $answer;
		
		if($data_format == 'rss')
		{
			$answer .= '<title>'.$status->post_message.'</title>';
			$answer .= '<description>'.$status->post_message.'</description>';
			$answer .= '<pubDate>'.gmdate('D, d M Y H:i:s \+0000',$status->post_date).'</pubDate>';
			$answer .= '<guid>'.$C->SITE_URL.'view/post:'.$status->post_id.'</guid>';
			$answer .= '<link>'.$C->SITE_URL.'view/post:'.$status->post_id.'</link>';
		}elseif($data_format == 'atom')
		{
			$answer .= '<title>'.$status->post_message.'</title>';
			$answer .= '<id>'.$C->SITE_URL.'1/statuses/show/'.$post_id.'</id>';
			$answer .= '<updated>'.date('Y-m-d\TH:i:s\Z',$status->post_date).'</updated>';
			$answer .= '<link type="text/html" href="'.$C->SITE_URL.'view/post:'.$post_id.'" rel="alternate"/>';
			$answer .= '<author><name>'.$status->post_user->username.'</name></author>';
		}
		
		return $answer;
	}
	function generate_header($format, $fancy_function=FALSE, $user_id = -1)
	{
		global $C;
		
		$header = '';
		
		if($format != 'xml' && $format != 'json' && $format != 'atom' && $format != 'rss') $format='xml';
		
		if($format == 'xml') $header = '<?xml version="1.0" encoding="utf-8"?'.'>';
		elseif($format == 'json') $header = ($fancy_function)? (($fancy_function!='?')? $fancy_function.'({':'({'):'{';		
		elseif($format == 'rss')
		{
			$header .= '<?xml version="1.0" encoding="UTF-8"'.'?'.'>';
			$header .= '<rss xmlns:atom="http://www.w3.org/2005/Atom" version="2.0">';
		}
		elseif($format == 'atom')
		{
			$header .= '<?xml version="1.0" encoding="utf-8"?'.'>';
			$header .= '<feed xml:lang="en-US" xmlns:georss="http://www.georss.org/georss" ';
			$header .= ' xmlns="http://www.w3.org/2005/Atom">';	
		}

		$header .= generate_desc($format, $user_id);
		
		return $header;
	}
	function generate_field($format, $field_name, $field_value)
	{
		if($format != 'xml' && $format != 'json' && $format != 'atom' && $format != 'rss') $format='xml';
		
		if($format == 'xml' || $format == 'rss' || $format == 'atom') $field = '<'.$field_name.'>'.$field_value.'</'.$field_name.'>';
		elseif($format == 'json') $field = '"'.$field_name.'": "'.$field_value.'",';

		return $field;
	}
	
	function generate_section($format, $field_name, $final = FALSE, $json_tag = FALSE)
	{
		if($format != 'xml' && $format != 'json' && $format != 'atom' && $format != 'rss') $format='xml';
		
		if($format == 'xml' || $format == 'rss' || $format == 'atom')
		{
			$field = (!$final)? '<'.$field_name.'>':'</'.$field_name.'>';
		}
		elseif($format == 'json')
		{
			if(!$final)
				$field = (!$json_tag) ? '"'.$field_name.'": {' : '["'.$field_name.'": {';
			else 
				$field = (!$json_tag) ? '}' : '}]';
			
		}

		return $field;
	}
	function generate_it_en($format, $final = FALSE)
	{
		$en = '';
		
		if($format == 'rss')
			$en .= ($final)? '</item>':'<item>';
		elseif($format == 'atom') 
			$en .= ($final)? '</entry>':'<entry>';
		
		return $en;
	} 
	function generate_desc($format, $user_id)
	{
		global $C;
		global $network;
		
		if($user_id != -1)
		{
			$u = $network->get_user_by_id($user_id);
			if(!$u) return false;
		}
		
		if($format == 'atom')
		{
			$dts	= @gmmktime(0, 0, 1, gmdate('m'), gmdate('d'), gmdate('Y'), $u->lastpost_date);
			
			if($user_id == -1)
			{
				$fld = '<title>'.$C->SITE_TITLE.' public timeline</title>';
  				$fld .= '<id>'.$C->SITE_URL.'1/stauses/public_timeline.atom</id>';
  				$fld .= '<link type="text/html" href="'.$C->SITE_URL.'1/public_timeline" rel="alternate"/>';
  				$fld .= '<link type="application/atom+xml" ';
				$fld .= ' href="'.$C->SITE_URL.'1/statuses/public_timeline.atom" rel="self"/>';
  				$fld .= '<updated>'.date('Y-m-d\TH:i:s\Z',time()).'</updated>';
				$fld .= '<subtitle>'.$C->SITE_TITLE.' updates from everyone!</subtitle>';
			}else
			{
				$fld = '<title>'.$C->SITE_TITLE.$u->username.'</title>';
				$fld .= '<id>'.$C->SITE_URL.'1/stauses/user_timeline/'.$u->id.'.atom</id>';
				$fld .= '<id> tag:'.$C->SITE_TITLE.','.date('Y', time()).':Status </id>';
				$fld .= '<link type="text/html" href="'.$C->SITE_URL.$u->username.'" rel="alternate"/>';
				$fld .= '<link type="application/atom+xml" ';
				$fld .= ' href="'.$C->SITE_URL.'1/statuses/user_timeline/'.$u->id.'.atom" rel="self"/>';
				$fld .= '<updated>'.date('Y-m-d\TH:i:s\Z',time()).'</updated>';
				$fld .= '<subtitle>'.$C->SITE_TITLE.' updates from '.$u->fullname.' / '.$u->username.'.</subtitle>';
			}
		}elseif($format == 'rss')
		{
			$fld = '<channel>';
			if($user_id != -1)
			{
				$fld .= '<title>'.$C->SITE_TITLE.$u->username.'</title>';
				$fld .= '<link>'.$C->SITE_URL.$u->username.'</link>';
				$fld .= '<atom:link type="application/rss+xml"'; 
				$fld .= ' href="'.$C->SITE_URL.'1/statuses/user_timeline/'.$u->id.'.rss" rel="self"/>';
				$fld .= '<description>'.$C->SITE_TITLE.' updates from '.$u->fullname.' / '.$u->username.'.</description>';
				$fld .= '<language>'.$u->language.'</language>';
			}else
			{
				$fld .= '<title>'.$C->SITE_TITLE.' public timeline</title>';
    				$fld .= '<link>'.$C->SITE_URL.'public_timeline</link>';
    				$fld .= '<atom:link type="application/rss+xml"';
				$fld .= ' href="'.$C->SITE_URL.'1/statuses/public_timeline.rss" rel="self"/>';
				$fld .= '<description>'.$C->SITE_URL.' public timeline</description>';
    				$fld .= '<language>en-us</language>';
			}
		}else $fld = '';
		
		return $fld;
	}
	function generate_error($format, $err, $link, $fancy_function=FALSE)
	{	
		global $C;
		$error = '';
		
		if($format != 'xml' && $format != 'json' && $format != 'atom' && $format != 'rss') $format='xml';

		if($format == 'xml')
		{
			$error .= '<?xml version="1.0" encoding="UTF-8" ?'.'>';
			$error .= '<hash>';
			$error .= '<request>'.$link.'</request>';
			$error .= '<error>'.$err.'</error>';
			$error .= '</hash>';			
		}elseif($format == 'json')
		{
			$error .= ($fancy_function && $fancy_function != '?')? $fancy_function.'(':'';
			$error .= ($fancy_function && $fancy_function == '?')? '(':'';
			$error .= '"hash":{';
			$error .= '"request": "'.$link.'",';
			$error .= '"error": "'.$err.'"';
			$error .= '}';
			if($fancy_function) $error .= ')';	
		}elseif($format == 'rss')
		{
			$error .= '<rss version="2.0">';
   			$error .= '<channel>';
   			
			$error .= '<title> '.$C->SITE_TITLE.' API Error </title> ';
			$error .= '<link>http://'.$C->SITE_URL.'/</link> ';
			$error .= '<description>Error message.</description> ';
			$error .= '<item>';
			
			$error .= '<request>'.$link.'</request>';
			$error .= '<error>'.$err.'</error>';
			
			$error .= '</item>';			
			$error .= '</channel></rss>';	
		}elseif($format == 'atom')
		{
			$error .= '<?xml version="1.0" encoding="utf-8"?'.'>';
 			$error .= '<feed xmlns="http://www.w3.org/2005/Atom">';
			$error .= '<link href="http://'.$C->SITE_URL.'/" />';
			$error .= '<id>urn:'.md5(time()).'</id>';
			$error .= '<author>';
			$error .= '<name>'.$C->SITE_URL.'</name>';
			$error .= '</author>';
			 
			 
			$error .= '<entry>';
   			
			$error .= '<title> '.$C->SITE_TITLE.' API Error </title> ';
			$error .= '<link>http://'.$C->SITE_URL.'/</link> ';
			$error .= '<description>Error message.</description> ';
			$error .= '<item>';
			
			$error .= '<request>'.$link.'</request>';
			$error .= '<error>'.$err.'</error>';
						
			$error .= '</entry></feed>';	
		}
		
		return $error;
	}
	function generate_bottom($format, $fancy_function=FALSE)
	{
		if($format == 'rss')
		{
			$bottom = '</channel></rss>';
		}elseif($format == 'atom')
		{
			$bottom = '</feed>';
		}elseif($format=='json')
		{
			$bottom = ($fancy_function)? '})':'}';
		}else $bottom = '';
		
		return $bottom;
	}
	function valid_fn($fn)
	{
		if($fn == '?') return true;
		else if(preg_match('/^([a-z0-9_]+)$/iu', $fn)) return true;
 		else return false;
	}
	
	function check_rate_limits($ip, $rate_num)
	{
		global $db2, $C;

		$res = $db2->query('SELECT 1 FROM ip_rates_limit WHERE ip=\''.ip2long($ip).'\' LIMIT 1');
		
		if($db2->num_rows($res) == 0)
		{
			$q = 'INSERT INTO ip_rates_limit(rate_limits, rate_limits_date, ip)';
			$q .= ' VALUES(0, '.intval(date('G', time())).', '.ip2long($ip).')';
			
			$res = $db2->query($q);
			if(!$db2->affected_rows($res)) return false; 
		}
		$q = 'SELECT rate_limits, rate_limits_date';
		$q .= ' FROM ip_rates_limit';
		$q .= ' WHERE ip = \''.ip2long($ip).'\' LIMIT 1';
		
		$res = $db2->query($q);
		$obj = $db2->fetch_object($res);
	
		if(!$obj) return false;		
	
		if( (($obj->rate_limits + $rate_num) < 150) || ($obj->rate_limits_date != date('G', time())))
		{
			if($obj->rate_limits_date != date('G', time()))
			{
				if(restart_rate_limits($ip)) return true;
					else return false;
			}
			
			if(update_rate_limits($ip, $rate_num)) return true;
				else return false;
				
		}else return false;			
	}
	function restart_rate_limits($ip)
	{
		global $db2;
		
		$q = 'UPDATE ip_rates_limit SET rate_limits=0, rate_limits_date=\''.intval(date('G', time())).'\'';
		$q .= ' WHERE ip = \''.ip2long($ip).'\' LIMIT 1';
		
		$res = $db2->query($q);
		
		if($res) return true;	
			else return false;			
	}
	function update_rate_limits($ip, $rate_num)
	{
		global $db2;
		
		$q = 'UPDATE ip_rates_limit';
		$q .= ' SET rate_limits=(rate_limits+'.$rate_num.'), rate_limits_date=\''.intval(date('G', time())).'\'';
		$q .= ' WHERE ip = \''.ip2long($ip).'\' LIMIT 1';
		
		$res = $db2->query($q);
		
		if($db2->affected_rows($res)) return true;	
			else return false;				
	}
?>